const coral = @import("coral");

const ext = @import("./ext.zig");

const file = @import("./file.zig");

const kym = @import("./kym.zig");

pub const Manifest = struct {
	title: [255:0]coral.io.Byte = [_:0]coral.io.Byte{0} ** 255,
	width: u16 = 640,
	height: u16 = 480,
	tick_rate: f32 = 60.0,

	pub fn load(self: *Manifest, env: *kym.RuntimeEnv, file_access: file.Access) kym.RuntimeError!void {
		const manifest_ref = try env.execute_file(file_access, file.Path.from(&.{"app.ona"})) orelse return;

		defer env.discard(manifest_ref);

		const width = @as(u16, get: {
			const ref = try kym.get_field(env, manifest_ref, "width");

			defer env.discard(ref);

			if (ref) |live_ref| {
				if (env.unbox(live_ref).expect_number()) |number| {
					if (number > 0 and number < coral.math.max_int(@typeInfo(@TypeOf(self.width)).Int)) {
						break: get @intFromFloat(number);
					}
				}
			}

			break: get self.width;
		});

		const height = @as(u16, get: {
			const ref = try kym.get_field(env, manifest_ref, "height");

			defer env.discard(ref);

			if (ref) |live_ref| {
				if (env.unbox(live_ref).expect_number()) |number| {
					if (number > 0 and number < coral.math.max_int(@typeInfo(@TypeOf(self.height)).Int)) {
						break: get @intFromFloat(number);
					}
				}
			}

			break: get self.height;
		});

		const tick_rate = @as(f32, get: {
			const ref = try kym.get_field(env, manifest_ref, "tick_rate");

			defer env.discard(ref);

			if (ref) |live_ref| {
				if (env.unbox(live_ref).expect_number()) |number| {
					break: get @floatCast(number);
				}
			}

			break: get self.tick_rate;
		});

		{
			const ref = try kym.get_field(env, manifest_ref, "title");

			defer env.discard(ref);

			const title_string = unbox: {
				if (ref) |live_ref| {
					if (env.unbox(live_ref).expect_string()) |string| {
						break: unbox string;
					}
				}

				break: unbox "";
			};

			const limited_title_len = @min(title_string.len, self.title.len);

			coral.io.copy(&self.title, title_string[0 .. limited_title_len]);
			coral.io.zero(self.title[limited_title_len .. self.title.len]);
		}

		self.tick_rate = tick_rate;
		self.width = width;
		self.height = height;
	}
};

pub fn log_info(message: []const coral.io.Byte) void {
	ext.SDL_LogInfo(
		ext.SDL_LOG_CATEGORY_APPLICATION,
		"%.*s",
		coral.math.clamped_cast(@typeInfo(c_int).Int, message.len),
		message.ptr,
	);
}

pub fn log_warn(message: []const coral.io.Byte) void {
	ext.SDL_LogWarn(
		ext.SDL_LOG_CATEGORY_APPLICATION,
		"%.*s",
		coral.math.clamped_cast(@typeInfo(c_int).Int, message.len),
		message.ptr,
	);
}

pub fn log_fail(message: []const coral.io.Byte) void {
	ext.SDL_LogError(
		ext.SDL_LOG_CATEGORY_APPLICATION,
		"%.*s",
		coral.math.clamped_cast(@typeInfo(c_int).Int, message.len),
		message.ptr,
	);
}
